#!/usr/bin/python

#	Copyright 2012 Jon Howell <jonh@jonh.net>
#
#    This file is part of shotgo.
#
#    shotgo is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    shotgo is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with shotgo.  If not, see <http://www.gnu.org/licenses/>.

# Installation instructions:
# sudo apt-get install python-mysqldb

import sys
import os
import stat
from ShotwellDB import ShotwellDB
from PiwigoDB import PiwigoDB

# My ancient server still has python 2.6 on it. Stupid squeeze.
#parser = argparse.ArgumentParser(
#	description="Convert shotwell metadata into piwigo metadata.")
#parser.add_argument('shotwell_db',
#	help='location of shotwell database')
#args = parser.parse_args()

class Args:
	def __init__(self, argv):
		self.hide_with_chmod = False
		self.shotwell_db = None

		self.parse(argv)

		if (self.shotwell_db==None):
			sys.stderr.write("Usage: %s <shotwell-db>\n" % self.prog)
			sys.stderr.write("  Converts shotwell metadata into piwigo metadata.\n")
			sys.exit(-1)

	def parse(self, argv):
		self.prog = argv[0]
		argv = argv[1:]
		while (len(argv)>0):
			if (argv[0]=='--hide-with-chmod'):
				self.hide_with_chmod = True
			elif (argv[0].startswith("-")):
				raise Exception("Unrecognized option: %s" % argv[0])
			else:
				if (self.shotwell_db!=None):
					raise Exception("Too many args")
				self.shotwell_db = argv[0]
			argv = argv[1:]

args = Args(sys.argv)
print "args: %s" % args.__dict__

# Adjust these settings:
class Configure: pass
config = Configure()

# Where to find shotwell's photo.db. This db will only be queried,
# not updated. (TODO pass "SQLITE_OPEN_READONLY" flag -- but I don't
# see how to do that from pysqlite.
config.shotwell_photo_db = os.path.expanduser(args.shotwell_db)
#	"~/.local/share/shotwell/data/photo.db")

# Photos with this shotwell tag won't be exposed to piwigo at all.
config.hide_tag = "hide"

# Photos with this shotwell tag will be available to everyone in piwigo.
config.public_tag = "public"

config.shotwell_photo_root = "/home/photos/"

# piwigo mysql database info
# THIS DATABASE WILL GET WIPED!
config.piwigo_host = "localhost"
config.piwigo_user = "piwigo"
config.piwigo_password = "piwigo"
config.piwigo_database = "piwigo"

config.piwigo_photo_root = "photo_root/"

class Export:
	def __init__(self, config, hide_with_chmod):
		self.config = config
		self.shotwell_db = ShotwellDB(config.shotwell_photo_db)

		if (hide_with_chmod):
			self.hide_with_chmod()
		else:
			self.convert()

	def hide_with_chmod(self):
		hide_tags = self.shotwell_db.fetch_tag(config.hide_tag)
		for int_id in hide_tags:
			photo = self.shotwell_db.select_photo_by_id(int_id)
			local_path = photo["filename"]
			print "plan to hide %s (now %o)" % (
				local_path, stat.S_IMODE(os.stat(local_path).st_mode))
			assert(os.path.exists(local_path))
			os.chmod(local_path, 0400)
		print "that would be %d photos" % len(hide_tags)

	def convert(self):
		self.piwigo_db = PiwigoDB(
			config.piwigo_host,
			config.piwigo_user,
			config.piwigo_password,
			config.piwigo_database,
			config.piwigo_photo_root)

		self.piwigo_db.clear()

		hide_tags = self.shotwell_db.fetch_tag(config.hide_tag)
		public_tags = self.shotwell_db.fetch_tag(config.public_tag)

		# A little cache
		id_to_md5sum = {}
		
		dbg_count = 0
		all_photo_rows = self.shotwell_db.select_all_photo_rows()
		for photo in all_photo_rows:
			if (photo["id"] in hide_tags):
				continue

			id_to_md5sum[photo["id"]] = photo["md5"]

			cat_path = self.shotwell_db.describe_event_as_category_path(photo)
			cat_id = self.piwigo_db.get_category(cat_path)
			local_path = photo["filename"]
			photo_path = self.strip_shotwell_prefix(local_path)
#			print "%-20s: %s" % (cat_path, local_path)
			rotation = self.convert_orientation(photo["orientation"])
			self.piwigo_db.insert_photo(local_path, photo_path, photo["md5"], cat_id, rotation, photo["id"] in public_tags)
			dbg_count += 1
			if ((dbg_count % 100)==0):
				sys.stderr.write("%5d/%5d images\n" % (dbg_count,len(all_photo_rows)))
#			if (dbg_count > 100):
#				break

		for tag in self.shotwell_db.select_tags():
			tag_name = tag["name"]
			for photo_id in self.shotwell_db.fetch_tag(tag_name):
				if (photo_id not in id_to_md5sum):
					# photo got hidden or something.
					continue
				self.piwigo_db.assign_tag(id_to_md5sum[photo_id], tag_name)

	def convert_orientation(self, orientation):
		# see shotwell src/Orientation.vala
		map = {
			1 : 0,
			6 : 3,
			3 : 2,
			8 : 1,
		}
		# the other four cases are flips; conversion not yet implemented.
		return map[orientation]

	def strip_shotwell_prefix(self, path):
		assert(path.startswith(self.config.shotwell_photo_root))
		return path[len(self.config.shotwell_photo_root):]

Export(config, args.hide_with_chmod)
